@@ -2,16 +2,18 @@
 """
 https://www.w3.org/TR/shacl/#sparql-constraints
 """
+
 from typing import Dict, List
 
 import rdflib
+from rdflib import URIRef
 
 from pyshacl.constraints.constraint_component import ConstraintComponent
 from pyshacl.consts import SH, SH_deactivated, SH_message, SH_select
 from pyshacl.errors import ConstraintLoadError, ValidationFailure
 from pyshacl.helper import get_query_helper_cls
-from pyshacl.pytypes import GraphLike
-
+from pyshacl.pytypes import GraphLike, SHACLExecutor
+from pyshacl.shape import Shape
 
 SH_sparql = SH.sparql
 SH_SPARQLConstraintComponent = SH.SPARQLConstraintComponent
@@ -26,7 +28,7 @@ class SPARQLBasedConstraint(ConstraintComponent):
 
     shacl_constraint_component = SH_SPARQLConstraintComponent
 
-    def __init__(self, shape):
+    def __init__(self, shape: Shape) -> None:
         super(SPARQLBasedConstraint, self).__init__(shape)
         sg = self.shape.sg.graph
         sparql_node_list = set(self.shape.objects(SH_sparql))
@@ -70,8 +72,7 @@ class SPARQLBasedConstraint(ConstraintComponent):
                 deactivated = next(iter(deactivated_node_list))
                 if not (isinstance(deactivated, rdflib.Literal) and isinstance(deactivated.value, bool)):
                     raise ConstraintLoadError(
-                        "SPARQLConstraintComponent value for sh:deactivated must be "
-                        "a Literal with type xsd:boolean.",
+                        "SPARQLConstraintComponent value for sh:deactivated must be a Literal with type xsd:boolean.",
                         "https://www.w3.org/TR/shacl/#SPARQLConstraintComponent",
                     )
                 deact = bool(deactivated.value)
@@ -82,15 +83,18 @@ class SPARQLBasedConstraint(ConstraintComponent):
         self.sparql_constraints = sparql_constraints
 
     @classmethod
-    def constraint_parameters(cls):
+    def constraint_parameters(cls) -> List[URIRef]:
         return [SH_sparql]
 
     @classmethod
-    def constraint_name(cls):
+    def constraint_name(cls) -> str:
         return "SPARQLConstraintComponent"
 
-    def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
+    def evaluate(
+        self, executor: SHACLExecutor, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List
+    ):
         """
+        :type executor: SHACLExecutor
         :type target_graph: rdflib.Graph
         :type focus_value_nodes: dict
         :type _evaluation_path: list
@@ -181,9 +185,9 @@ class SPARQLBasedConstraint(ConstraintComponent):
                 v = var_dict.pop('value', None)
                 t = var_dict.pop('this', None)
                 if (p is not None) or (v is not None) or (t is not None):
-                    if (t, p, v) in dedup_set:
+                    # Guard against duplicate results
+                    if (t, p, v, var_dict) in violations:
                         continue
                     violations.append((t, p, v, var_dict))
-                    dedup_set.add((t, p, v))
 
         return violations
