@@ -4,10 +4,11 @@ from functools import wraps
 from typing import Iterator, List, Optional, Tuple, Union, cast
 
 import rdflib
-
 from rdflib.namespace import NamespaceManager
 
-from .consts import SH, RDF_first, RDFNode
+from .consts import OWL, SH, RDF_first, RDFNode
+
+OWLsameAs = OWL.sameAs
 
 
 def with_dict_cache(f):
@@ -32,7 +33,7 @@ def stringify_blank_node(
         raise RuntimeError("Can only stringify a blank node when graph is a rdflib.Graph")
     assert isinstance(graph, rdflib.Graph)
     assert isinstance(bnode, rdflib.BNode)
-    if recursion >= 9:
+    if recursion >= 12:
         return "<http://recursion.too.deep>"
     stringed_cache_key = id(graph), str(bnode)
 
@@ -45,7 +46,7 @@ def stringify_blank_node(
         ns_manager = graph.namespace_manager
         ns_manager.bind("sh", SH)
 
-    def stringify_list(node):
+    def stringify_list(node: rdflib.BNode) -> str:
         nonlocal graph, ns_manager, recursion
         item_texts: List[str] = []
         for item in iter(graph.items(node)):
@@ -70,8 +71,12 @@ def stringify_blank_node(
             continue
         o_texts = []
         for o in objs:
-            o_text = stringify_node(graph, o, ns_manager=ns_manager, recursion=recursion + 1)
-            o_texts.append(o_text)
+            if p is OWLsameAs and o is bnode:
+                # Avoid a crazy owl:sameAs recursion with self.
+                o_texts.append("<self>")
+            else:
+                o_text = stringify_node(graph, o, ns_manager=ns_manager, recursion=recursion + 1)
+                o_texts.append(o_text)
         if len(o_texts) > 1:
             o_texts.sort()
             o_text = ", ".join(o_texts)
@@ -90,15 +95,19 @@ def stringify_blank_node(
 
 
 def stringify_literal(graph: rdflib.Graph, node: rdflib.Literal, ns_manager: Optional[NamespaceManager] = None):
-    lit_val_string = str(node.value)
-    lex_val_string = str(node)
+    lit_val_string: Union[str, None] = None if node.value is None else str(node.value)
+    lex_string = str(node)
     if ns_manager is None:  # pragma: no cover
         ns_manager = graph.namespace_manager
         ns_manager.bind("sh", SH)
-    if lit_val_string != lex_val_string:
-        val_string = "\"{}\" = {}".format(lex_val_string, lit_val_string)
+    if lit_val_string is not None:
+        i_at = lit_val_string.find(" object at 0x")
+        if i_at > 0:
+            lit_val_string = lit_val_string[:i_at]
+    if lit_val_string is not None and lit_val_string != lex_string:
+        val_string = "\"{}\" = {}".format(lex_string, lit_val_string)
     else:
-        val_string = "\"{}\"".format(lex_val_string)
+        val_string = "\"{}\"".format(lex_string)
     if node.language:
         lang_string = ", lang={}".format(str(node.language))
     else:
@@ -115,7 +124,7 @@ def stringify_literal(graph: rdflib.Graph, node: rdflib.Literal, ns_manager: Opt
     return node_string
 
 
-def find_node_named_graph(dataset, node):
+def find_node_named_graph(dataset: Union[rdflib.Dataset, rdflib.ConjunctiveGraph], node) -> rdflib.Graph:
     """
     Search through each graph in a dataset for one node, when it finds it, returns the graph it is in
     :param dataset:
@@ -124,14 +133,26 @@ def find_node_named_graph(dataset, node):
     """
     if isinstance(node, rdflib.Literal):
         raise RuntimeError("Cannot search for a Literal node in a dataset.")
-    for g in iter(dataset.contexts()):
-        try:
-            # This will issue StopIteration if node is not found in g, and continue to the next graph
-            _ = next(iter(g.predicate_objects(node)))
-            return g
-        except StopIteration:
+
+    # Check if node is a subject in any graph
+    for q in iter(dataset.quads((node, None, None, None))):
+        s, p, o, g = q
+        if g is None:
             continue
-    raise RuntimeError("Cannot find that node in any named graph.")
+        elif isinstance(g, rdflib.Graph):
+            return g
+        else:
+            return dataset.get_context(g)
+    # Now check if node is a object in any graph
+    for q in iter(dataset.quads((None, None, node, None))):
+        s, p, o, g = q
+        if g is None:
+            continue
+        elif isinstance(g, rdflib.Graph):
+            return g
+        else:
+            return dataset.get_context(g)
+    raise LookupError(f"Cannot find node {node} in any named graph.")
 
 
 def stringify_node(
@@ -139,7 +160,7 @@ def stringify_node(
     node: RDFNode,
     ns_manager: Optional[Union[NamespaceManager, rdflib.Graph]] = None,
     recursion: int = 0,
-):
+) -> str:
     if ns_manager is None:
         ns_manager = graph.namespace_manager
     if isinstance(ns_manager, rdflib.Graph):
@@ -166,8 +187,9 @@ def stringify_node(
 
 def stringify_graph(graph: rdflib.Graph):
     string_builder = ""
+    t: Tuple[rdflib.term.Node, rdflib.term.Node, rdflib.term.Node]
     for t in iter(graph):
-        n1, n2, n3 = t  # type: Tuple[rdflib.term.Node, rdflib.term.Node, rdflib.term.Node]
+        n1, n2, n3 = t
         node_string = stringify_node(graph, n1, ns_manager=graph.namespace_manager)
         string_builder += node_string + ", "
         node_string = stringify_node(graph, n2, ns_manager=graph.namespace_manager)
