@@ -1,11 +1,10 @@
 # HTTP Server for PySHACL
 import os
 import sys
-
+from typing import Union
 
 try:
     import sanic.application.logo
-
     from sanic import Request, Sanic
     from sanic.exceptions import InvalidUsage
     from sanic.response import HTTPResponse, JSONResponse, text
@@ -25,7 +24,6 @@ from . import __version__ as pyshacl_version
 from . import validate
 from .errors import ConstraintLoadError, ReportableRuntimeError, RuleLoadError, ShapeLoadError, ValidationFailure
 
-
 API_VERSION = "v1"
 
 
@@ -47,19 +45,19 @@ class InputRDFFormat(Enum):
 
 @dataclass
 class ValidationRequest:
-    data_graph: openapi_types.String(
+    data_graph: openapi_types.String(  # type: ignore[valid-type]
         required=True, title="DataGraph", description="The target DataGraph to validate (serialized in an RDF string)"
     )
-    shapes_graph: openapi_types.String(
+    shapes_graph: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=True,
         title="DataGraph",
         description="Your SHACL ShapesGraph (serialized in an RDF string)",
     )
-    ontology_graph: openapi_types.String(
+    ontology_graph: openapi_types.String(  # type: ignore[valid-type]
         required=False, nullable=True, title="OntologyGraph", description="Optional ontological definitions graph"
     )
-    data_graph_format: openapi_types.String(
+    data_graph_format: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default="auto",
@@ -67,7 +65,7 @@ class ValidationRequest:
         description="Optionally specify the RDF format for your DataGraph",
         enum=InputRDFFormat,
     )
-    shapes_graph_format: openapi_types.String(
+    shapes_graph_format: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default="auto",
@@ -75,7 +73,7 @@ class ValidationRequest:
         description="Optionally specify the RDF format for your ShapesGraph",
         enum=InputRDFFormat,
     )
-    ontology_graph_format: openapi_types.String(
+    ontology_graph_format: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default="auto",
@@ -83,14 +81,14 @@ class ValidationRequest:
         description="Optionally specify the RDF format for your OntologyGraph",
         enum=InputRDFFormat,
     )
-    advanced: openapi_types.Boolean(
+    advanced: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
         title="Advanced",
         description="Enable features from the SHACL Advanced Features spec.",
     )
-    inference: openapi_types.String(
+    inference: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default="none",
@@ -98,14 +96,14 @@ class ValidationRequest:
         description="Set a pre-inference option",
         enum=InferenceKind,
     )
-    do_owl_imports: openapi_types.Boolean(
+    do_owl_imports: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
         title="Do OWL Imports",
         description="Enable the feature to follow links to import OWL ontologies in Shapes graph and Ontology Graph.",
     )
-    allow_infos: openapi_types.Boolean(
+    allow_infos: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
@@ -113,7 +111,7 @@ class ValidationRequest:
         description="The datagraph will still be considered conformant when encountering constraint failures with "
         "sh:Info level severity.",
     )
-    allow_warnings: openapi_types.Boolean(
+    allow_warnings: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
@@ -121,7 +119,7 @@ class ValidationRequest:
         description="The datagraph will still be considered conformant when encountering constraint failures with "
         "sh:Warning or sh:Info level severity.",
     )
-    iterate_rules: openapi_types.Boolean(
+    iterate_rules: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
@@ -129,10 +127,10 @@ class ValidationRequest:
         description="Continue to execute SHACL Rules until the resulting output graph reaches steady state. "
         "This only works when advanced mode is enabled, and is usually not required.",
     )
-    js: openapi_types.String(
+    js: openapi_types.String(  # type: ignore[valid-type]
         required=False, nullable=False, default=False, title="JS", description="Enable SHACL-JS validator extension"
     )
-    metashacl: openapi_types.Boolean(
+    metashacl: openapi_types.Boolean(  # type: ignore[valid-type]
         required=False,
         nullable=False,
         default=False,
@@ -155,16 +153,16 @@ failures_array_ref = openapi.Component(
 
 @dataclass
 class ValidationResponseSimple:
-    conforms: openapi_types.Boolean(
+    conforms: openapi_types.Boolean(  # type: ignore[valid-type]
         required=True, title="Conforms", description="The datagraph conforms to the SHACL Shapes"
     )
-    validation_report: openapi_types.String(
+    validation_report: openapi_types.String(  # type: ignore[valid-type]
         required=False,
         nullable=True,
         title="ValidationReport",
         description="The text representation of the validation report.",
     )
-    validation_failures: failures_array_ref
+    validation_failures: failures_array_ref  # type: ignore[valid-type]
 
 
 def make_validation_response_RDF(format, mimetype):
@@ -434,7 +432,7 @@ def run_server():
     )
 
 
-def cli():
+def main(prog: Union[None, str] = None) -> None:
     """CLI entrypoint for the HTTP Service"""
     try:
         run_server()
@@ -444,5 +442,5 @@ def cli():
         sys.stderr.write(f"{r}\r\n")
         traceback.print_tb(r.__traceback__, file=sys.stderr)
         sys.stderr.flush()
-        return 1
-    return 0
+        sys.exit(1)
+    sys.exit(0)
