@@ -60,7 +60,7 @@ exOnt:hasPet a rdf:Property ;
     rdfs:domain exOnt:Human ;
     rdfs:range exOnt:Pet .
 
-exOnt:nlegs a rdf:Property ;
+exOnt:nLegs a rdf:Property ;
     rdfs:domain exOnt:Animal ;
     rdfs:range exOnt:integer .
 
@@ -103,18 +103,29 @@ exShape:AnimalShape a sh:NodeShape ;
     sh:targetClass exOnt:Animal .
 """
 
+
 def test_062():
     data_fp = BytesIO(data)
     shacl_fp = BytesIO(shacl_data)
     ont_fp = BytesIO(ont_data)
     try:
-        conforms, g, s = validate(data_fp, shacl_graph=shacl_fp, ont_graph=ont_fp, data_graph_format="json-ld",
-                                  shacl_graph_format="turtle", ont_graph_format="turtle", abort_on_first=False,
-                                  meta_shacl=False, debug=True, advanced=True)
+        conforms, g, s = validate(
+            data_fp,
+            shacl_graph=shacl_fp,
+            ont_graph=ont_fp,
+            data_graph_format="json-ld",
+            shacl_graph_format="turtle",
+            ont_graph_format="turtle",
+            abort_on_first=False,
+            meta_shacl=False,
+            debug=True,
+            advanced=True,
+        )
     except Exception as e:
         print(e)
         raise
     assert conforms
 
+
 if __name__ == "__main__":
     test_062()
